/* 
 * Copyright (c) 2000 Mike Muir
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	Parts of this code derived from examples by Takanori Watanabe
 *	Makefile.in and configure.in based on Chris D. Faulhaber's wmlmmon files.
 * 
 * Misc Display functions for wmhm.
 */

#include "display.h"

void Usage();
void UpdateScaleLabel(int scale);
void DisplayVoltages(float o_volts);
void DisplayCPUTemps(short order, float temp);
void AdjustVoltageLabels(unsigned short int data_display, unsigned short int voltage_type);
void Alarm(short odd_test, int temp, int tbar_colormap);
void PlaySpeaker(char *tune_array);

void Usage() 
{
  	printf("wmhm %s\n\n",WMHM_VERSION);
  	(void)fprintf(stderr, "%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n",
		"usage: wmhm [options]",
		"	-I | -S			/dev/io method | SMBUS method (if supported)",
		"	-C			Start with CPU Temp display rather than Fans.",
		"	-f      		Start with Fahrenheit temperature scale.",
		"	-k			Start with Kelvin temperature scale.",
		"	-m #                    Start with colourmap #",
		"                                0=Tropical 1=Ice 2=Greenman",
		"                                [default: Tropical]",
		"	-h      		Display this help screen.",
		"	-d #			Number of microseconds to wait between",
		"				 screen updates [default: 1000000]",
		"	-a #			Temperature Alarm High Threshold (\260C)",
		"	-v #			Core Voltage (V) [default: 2V]",
		"	-p #			Voltage Warning Margin (%) [default: 10]",
		"	-t [1,2,3,4,5,6]	Voltage Type (Core,IO,+5,+12,-12,-5)",
		"				 [default: Core]");
  	exit(1);
}

void UpdateScaleLabel(int scale)
{  
  	switch (scale) {
  		case CELCIUS:
    		copyXPMArea(61,74,5,7,27,6);		
    		break;
  		case FAHRENHEIT:
    		copyXPMArea(61,65,5,7,27,6);
    		break;
  		case KELVIN:
    		copyXPMArea(71,65,5,7,27,6);
  	}
}

void DisplayVoltages(float o_volts)
{
	
  	int c;
  	int digit;
  	c = abs((int)(o_volts * 10));
  	if (c > 0) {
    	digit = c % 10;
    	copyXPMArea((digit<<2)+(digit<<1),64,6,9,45,33);
    	if (c > 9) {
      	digit = ((c % 100) - digit) / 10;
      	copyXPMArea((digit<<2)+(digit<<1),64,6,9,39,33);
      	if (c > 99) {
				digit = ((c % 1000) - digit) / 100;
				copyXPMArea((digit<<2)+(digit<<1),64,6,9,33,33);
      	}
   	}
  	}
  	copyXPMArea(35,73,1,1,45,40); // Decimal point for all voltages.
}

void DisplayCPUTemps(short order, float temp)
{
	int c;
        int digit;
        c = abs((int)temp);
        if (c > 0) {
        digit = c % 10;
        copyXPMArea((digit<<2)+(digit<<1),64,6,9,45,((order-1)*8)+33);
        if (c > 9) {
        digit = ((c % 100) - digit) / 10;
        copyXPMArea((digit<<2)+(digit<<1),64,6,9,39,((order-1)*8)+33);
        if (c > 99) {
                                digit = ((c % 1000) - digit) / 100;
                                copyXPMArea((digit<<2)+(digit<<1),64,6,9,33,((order-1)*8)+33);
        }
        }
        }  else if (c == 0) {
                   copyXPMArea(0, 64, 6, 9, 45,((order-1)*8)+33);
        }
}
void AdjustVoltageLabels(unsigned short int data_display, unsigned short int voltage_type)
{
	switch (data_display) {
	case 2:
				copyXPMArea(24,109,23,7,34,23); // CPUS Text
				copyXPMArea(7,65,5,7,26,34);  // 1
                                copyXPMArea(13,65,5,7,26,42); // 2
                                copyXPMArea(35,73,1,3,32,36); // : for the 1
                                copyXPMArea(35,73,1,3,32,44); // : for the 2
                                break;
      	case D_FANS:
				copyXPMArea(0,93,23,7,34,23); // FANS Text
				copyXPMArea(7,65,5,7,26,34);  // 1
				copyXPMArea(13,65,5,7,26,42); // 2
				copyXPMArea(19,65,5,7,26,50); // 3
				copyXPMArea(35,73,1,3,32,36); // : for the 1
				copyXPMArea(35,73,1,3,32,44); // : for the 2
				copyXPMArea(35,73,1,3,32,52); // : for the 3
				break;
      	case 1:
				copyXPMArea(36,93,5,7,52,34); // 'V' Beside Reading
				switch (voltage_type) {
					case D_CORE:
						copyXPMArea(0,101,23,7,34,23); // CORE Text
					  	copyXPMArea(71,55,5,7,26,34);  // no 1
					  	copyXPMArea(71,55,5,7,26,42);  // no 2
					  	copyXPMArea(71,55,5,7,26,50);  // no 3
					  	copyXPMArea(32,25,1,3,32,36);  // no : for the 1
					  	copyXPMArea(32,25,1,3,32,44);  // no : for the 2
					  	copyXPMArea(32,25,1,3,32,52);  // no : for the 3
					  	break;
					case D_IO:
					  	copyXPMArea(0,109,23,7,34,23); // IO Text
					  	copyXPMArea(71,55,5,7,26,34);  // no 1
					  	copyXPMArea(71,55,5,7,26,42);  // no 2
					  	copyXPMArea(71,55,5,7,26,50);  // no 3
					  	copyXPMArea(32,25,1,3,32,36);  // no : for the 1
					  	copyXPMArea(32,25,1,3,32,44);  // no : for the 2
					  	copyXPMArea(32,25,1,3,32,52);  // no : for the 3
	  					break;
					case D_P5:
					  	copyXPMArea(24,93,23,7,34,23); // 5 Text
					  	copyXPMArea(71,55,5,7,26,34);  // no 1
					  	copyXPMArea(71,55,5,7,26,42);  // no 2
					  	copyXPMArea(71,55,5,7,26,50);  // no 3
					  	copyXPMArea(32,25,1,3,32,36);  // no : for the 1
					  	copyXPMArea(32,25,1,3,32,44);  // no : for the 2
					  	copyXPMArea(32,25,1,3,32,52);  // no : for the 3
					  	break;
					case D_P12:
					  	copyXPMArea(24,101,23,7,34,23);// 12 Text
					  	copyXPMArea(71,55,5,7,26,34);  // no 1
					  	copyXPMArea(71,55,5,7,26,42);  // no 2
					  	copyXPMArea(71,55,5,7,26,50);  // no 3
					  	copyXPMArea(32,25,1,3,32,36);  // no : for the 1
					  	copyXPMArea(32,25,1,3,32,44);  // no : for the 2
					  	copyXPMArea(32,25,1,3,32,52);  // no : for the 3
					  	break;
					case D_N12:
					  	copyXPMArea(24,101,23,7,34,23); // 12 Text
					  	copyXPMArea(35,73,3,1,27,26);  // -
					  	copyXPMArea(71,55,5,7,26,34);  // no 1
					  	copyXPMArea(71,55,5,7,26,42);  // no 2
					  	copyXPMArea(71,55,5,7,26,50);  // no 3
					  	copyXPMArea(32,25,1,3,32,36);  // no : for the 1
					  	copyXPMArea(32,25,1,3,32,44);  // no : for the 2
					  	copyXPMArea(32,25,1,3,32,52);  // no : for the 3
					  	break;
					case D_N5:
					  	copyXPMArea(24,93,23,7,34,23);// 5 Text
					  	copyXPMArea(35,73,3,1,27,26);  // - 
					  	copyXPMArea(71,55,5,7,26,34);  // no 1
					  	copyXPMArea(71,55,5,7,26,42);  // no 2
					  	copyXPMArea(71,55,5,7,26,50);  // no 3
					  	copyXPMArea(32,25,1,3,32,36);  // no : for the 1
					  	copyXPMArea(32,25,1,3,32,44);  // no : for the 2
					  	copyXPMArea(32,25,1,3,32,52);  // no : for the 3
					  	break;
				}
				break;
	}
}

void Alarm(short odd_test, int temp, int tbar_colormap)
{
	int c;
	char tune_array[] = "abcdef";
	PlaySpeaker(tune_array);
	c = (int)((temp / 50)*21);
	if (odd_test) {
	   if (!tbar_colormap)
	      copyXPMArea(0,83,22,10,37,5);  // Tropical background		
	   else
              copyXPMArea(48,92,22,10,37,5); // Ice/Green Man Background
	} else {
	   switch (tbar_colormap) {
                 case 0:
                    copyXPMArea(1,73,c,9,38,5);    // Copy Tropical bar
                    break;
                 case 1:
                    copyXPMArea(48,82,c,9,38,5);   // Copy Ice bar
                    break;
                 case 2:
                    copyXPMArea(48,102,c,9,38,5);  // Copy Green Man bar
                    break;
              }	
	}
}

	
