/*
 *
 * Copyright (c) 2000 Mike Muir
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	Makefile.in and configure.in based on Chris D. Faulhaber's wmlmmon files.
 *
 */

#include "wmhm.h"

int OpenIO();
int ReadByte(u_char *return_value, int addr);
int WriteByte(int addr, int value);

void Usage();
void UpdateScaleLabel(int scale);
void DisplayVoltages(float volts_original);
void DisplayCPUTemps(short order, float temp);
void AdjustVoltageLabels(unsigned short int data_display, unsigned short int voltage_type);
void Alarm(short odd_test, int temp, int tbar_colormap);

int main (int argc, char **argv)
{
  	XEvent Event;

  	short scale = CELCIUS;
  	short tbar_colormap = TBAR_COLORMAP;
  	short odd_test = FALSE;
	short sound_alarm = FALSE;
	short ignore = FALSE;
	short mb_limit = MB_LIMIT;
  	int delay = DELAY;
  	int digit = 0;
  	int c, i;
	int cpu0_temp;
	int cpu1_temp;
  	int array_element = 0;
  	char warn_percent = RANGE_P;
  	char sw_char;
  	int fandiv[3];
  	int display_changed = TRUE;
  	short clicked_region;
  	float volts;
  	float o_volts;
  	float volts_diff;
  	float volts_range;
  	float user_v_core = CORE_VOLTAGE;
	float user_v_core_set = CORE_VOLTAGE;
  	double mb_temp;
  	unsigned short int voltage_type = D_CORE;
  	unsigned short int data_display = START_DISPLAY;
  	unsigned char smb_return = 0, cpu0_return = 0, cpu1_return = 0; 
	float mb_return = 0.0;
  	const double vfactor[]={1,1,1,1.67,4,-4,-1.67};

#ifdef HAVE_SMBUS 

  	struct smbcmd cmd;
  	int isa_cmd;

#else

  	int isa_cmd;

#endif
  	u_char return_value;
  	u_char return_value2;
  	int file_handle;

  // Command line args

	while ((sw_char = getopt(argc, argv, "S C I f k h a:d:v:p:t:m:")) != -1)
		switch (sw_char) {
		case 'a':
		mb_limit = atoi(optarg);
		printf("Temperature Alarm Threshold set to %d\260 C.\n", mb_limit);
		break;
      		case 'C':
      		printf("Starting with CPU Temp display.\n");
		data_display = 2;
		display_changed = TRUE;
      		break;
    		case 'I':
      		printf("Using ISA Method.\n");
      		SMBUS = FALSE;
      		break;
    		case 'S':
#ifdef HAVE_SMBUS
      		printf("Using SMBUS Method.\n");
      		SMBUS = TRUE;
#else
      		printf("Not compiled for use with SMBUS, reverting to ISA Method.\n");
      		SMBUS = FALSE;
#endif
      		break;
    		case 'f':
      		printf("Using Fahrenheit scale.\n");
      		scale = FAHRENHEIT;
      		break;
    		case 'k':
      		printf("Using Kelvin scale.\n");
      		scale = KELVIN;
      		break;
    		case 'h':
      		Usage();
      		break;
    		case 'd':
      		delay = atoi(optarg);
      		break;
    		case 'v':
      		user_v_core = atof(optarg);
		user_v_core_set = user_v_core;
      		printf("User Core Voltage: %g.\n",user_v_core);
      		break;
    		case 'p':
      		warn_percent = atoi(optarg);
      		printf("Voltage Warning Delta: %d%%.\n",warn_percent);
      		break;
    		case 't':
      		voltage_type = atoi(optarg);
      		printf("Using Voltage Type: %d.\n",voltage_type);
      		break;
    		case 'm':
      		tbar_colormap = atoi(optarg);
      		printf("Starting with colourmap %d.\n",tbar_colormap);
      		break;
    		default:
    			break;
    	}	
  
  	file_handle = OpenIO();
	{	
  	static const int fandivide[]={1,2,4,8};

  	ReadByte(&return_value, 0x47);
  	smb_return = return_value;
	
  	fandiv[0]=fandivide[(smb_return>>4)&3];
  	fandiv[1]=fandivide[(smb_return>>6)&3];
  	fandiv[2]=2;
	}
#ifdef HAVE_SMBUS
  	cmd.data.byte_ptr=&smb_return;
#endif

  	openXwindow(argc, argv, wmhm_master_xpm, wmhm_mask_bits, wmhm_mask_width, wmhm_mask_height);

  	UpdateScaleLabel(scale);	

  	AddMouseRegion(0, 5, 5, 32, 14);
  	AddMouseRegion(1, 5, 19, 19, 58);
  	AddMouseRegion(2, 24, 19, 58, 58);
  	AddMouseRegion(3, 37, 5, 58, 13);

	while (1) {
   	while(XPending(display)) {
   		XNextEvent(display,&Event);
			switch (Event.type) {
				case Expose:
					RedrawWindow();
					break;
      		case DestroyNotify:
					XCloseDisplay(display);
					close(file_handle);
					exit(0);
					break;
   	   	case ButtonPress:
					clicked_region = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
				switch (clicked_region) {
					case 0:
	  					scale = scale < 2 ? scale + 1 : 0;
	  					UpdateScaleLabel(scale);
	  					break;
					case 1:
	  					voltage_type = voltage_type < 6 ? voltage_type + 1 : 1;
	  					display_changed = TRUE;
	  					break;
					case 2:
	  					data_display = data_display < 2 ? data_display + 1 : 0;
	  					display_changed = TRUE;
	  					break;
					case 3:
						sound_alarm = FALSE;
						ignore = ignore < 1 ? ignore + 1 : 0;
	  					tbar_colormap = tbar_colormap < 2 ? tbar_colormap + 1 : 0;
	  					break;
					default:
	  					break;
				}  
     		}
 		}

   	// Clear the Fan and Temp numbers at the start of each refresh.

  		copyXPMArea(71,55,5,7,26,23); // Sign LCD Digit in top left of Data Display.

  		copyXPMArea(71,55,5,7,6,6);   // Temp Digits
  		copyXPMArea(71,55,5,7,12,6);
	  	copyXPMArea(71,55,5,7,18,6);

  		copyXPMArea(71,55,5,7,34,34); // Fan and Voltage Reading Digits
  		copyXPMArea(71,55,5,7,40,34);
  		copyXPMArea(71,55,5,7,46,34);
  		copyXPMArea(71,55,5,7,52,34);

  		copyXPMArea(71,55,5,7,34,42);
  		copyXPMArea(71,55,5,7,40,42);
  		copyXPMArea(71,55,5,7,46,42);
  		copyXPMArea(71,55,5,7,52,42);

  		copyXPMArea(71,55,5,7,34,50);
 		copyXPMArea(71,55,5,7,40,50);
  		copyXPMArea(71,55,5,7,46,50);
   		copyXPMArea(71,55,5,7,52,50);

	   // Reset Data Display dependant on which display is selected.

	   if (display_changed)
	   	AdjustVoltageLabels(data_display, voltage_type);

	   // Voltage Meter

	   switch (voltage_type) {
	   case 1:
	#ifdef HAVE_SMBUS 
	  		cmd.cmd=0x20;
	      isa_cmd=0x20;
	#else
	      isa_cmd=0x20;
	#endif
	  		array_element=0;
			user_v_core = user_v_core_set;
	  		break;
	   case 2:
	#ifdef HAVE_SMBUS 
	      cmd.cmd=0x22;
	      isa_cmd=0x22;
	#else
	      isa_cmd=0x22;
	#endif
	      array_element=2;
	      user_v_core = 3.3;
	      break;
	   case 3:
	#ifdef HAVE_SMBUS 
	      cmd.cmd=0x23;
	      isa_cmd=0x23;
	#else
	      isa_cmd=0x23;
	#endif
	      array_element=3;
	      user_v_core = 5;
	      break;
	   case 4:
	#ifdef HAVE_SMBUS 
	      cmd.cmd=0x24;
	      isa_cmd=0x24;
	#else
	      isa_cmd=0x24;
	#endif
	      array_element=4;
	      user_v_core = 12;
	      break;
	   case 5:
	#ifdef HAVE_SMBUS 
	      cmd.cmd=0x25;
	      isa_cmd=0x25;
	#else
	      isa_cmd=0x25;
	#endif
	      array_element=5;
	      user_v_core = -12;
	      break;
	   case 6:
	#ifdef HAVE_SMBUS 
	      cmd.cmd=0x26;
	      isa_cmd=0x26;
	#else
	      isa_cmd=0x26;
	#endif
	      array_element=6;
	      user_v_core = -5;
	      break;
	   default:
	#ifdef HAVE_SMBUS 
	      cmd.cmd=0x20;
	      isa_cmd=0x20;
	#else
	      isa_cmd=0x20;
	#endif
	      array_element=0;
	   }

	   ReadByte(&return_value, isa_cmd);
	   smb_return = return_value;

	   volts = 0;
	   volts = (float)smb_return*vfactor[array_element]/64;
	   o_volts = volts;
	   copyXPMArea(22,73,13,17,6,20);
	   copyXPMArea(22,73,13,17,6,40);

	   volts_diff = 0;
	   volts_range = user_v_core * ((float)warn_percent/100);
	   if (volts_range < 0) 
	   	volts_range *= -1;
	   volts_diff = volts-user_v_core;
	   volts = (float)volts_diff/volts_range;

	   if (volts > 1) volts = 1;
	   if (volts < -1) volts = -1;

	   c = rint(volts*9);

	   if (c > 0) copyXPMArea(35,73,13,abs(c<<1),6,(38-(c<<1)));
	   if (c < 0) copyXPMArea(35,73,13,abs(c<<1),6,40);

	   // Motherboard Temperature (LM79/78 onchip thermistors)

	   ReadByte(&return_value, 0x27);

#ifdef HAVE_BP6
           cpu1_return = return_value;
#else
           mb_return = return_value;
#endif

	   // CPU_0 Temperature

           WriteByte(0x4E,0x01);
           ReadByte(&return_value, 0x50);
           ReadByte(&return_value2, 0x51);
           if (return_value2 == 128) 
  	      return_value += 0.5;
           WriteByte(0x4E,0);
           if (return_value > 200) 
	      return_value = 0.0;
           cpu0_return = return_value;
 
	   // CPU_1 Temperature

           WriteByte(0x4E,0x02);
           ReadByte(&return_value, 0x50);
           ReadByte(&return_value2, 0x51);
           if (return_value2 == 128) 
	      return_value += 0.5;
           WriteByte(0x4E,0);
           if (return_value > 200) 
	      return_value = 0.0;

#ifdef HAVE_BP6
              mb_return = return_value;
#else
              cpu1_return = return_value;
#endif

	   switch (scale) {
              case FAHRENHEIT:
                 mb_temp = (int)(mb_return*1.8) + 32;
                 cpu0_temp = (int)(cpu0_return*1.8) + 32;
                 cpu1_temp = (int)(cpu1_return*1.8) + 32;                 
                 break;
              case KELVIN:
                 mb_temp = (int)(mb_return+273); 
                 cpu0_temp = (int)(cpu0_return+273);
                 cpu1_temp = (int)(cpu1_return+273);
                 break;
              default:
                 mb_temp = (int)mb_return;
                 cpu0_temp = (int)cpu0_return;
                 cpu1_temp = (int)cpu1_return;
           }

           c = mb_temp;

		if (c > 0) {
	           digit = c % 10;
	           copyXPMArea((digit<<2)+(digit<<1),64,6,9,17,5);
	           if (c > 9) {
				digit = ((c%100) - digit) / 10;
				copyXPMArea((digit<<2)+(digit<<1),64,6,9,11,5);
				if (c > 99) {
		  			digit = (c - (c % 100)) / 100;
		  			copyXPMArea((digit<<2)+(digit<<1),64,6,9,5,5);
				}
	           }
	        } else if (c == 0) {
		   copyXPMArea(0, 64, 6, 9, 17, 5);
		}
	   
	   // Temperature Meter

	   if (!tbar_colormap)
	      copyXPMArea(0,83,22,10,37,5);  // Tropical background
	   else 
  	      copyXPMArea(48,92,22,10,37,5); // Ice/Green Man Background
	   if (mb_return > (mb_limit-1)) {
	      mb_return = mb_limit;
	      sound_alarm = TRUE;
	   } else 
	      sound_alarm = FALSE;
	   c = (int)((mb_return / TEMP_LIMIT)*21);
	      switch (tbar_colormap) {
	         case 0:
		    copyXPMArea(1,73,c,9,38,5);    // Copy Tropical bar
		    break;
		 case 1:
		    copyXPMArea(48,82,c,9,38,5);   // Copy Ice bar
		    break;
		 case 2:
		    copyXPMArea(48,102,c,9,38,5);  // Copy Green Man bar
		    break;
	      }

	   // Fan Guages

	   if (data_display == D_FANS) {
	    	for(i=0x28;i<0x2b;i++){
				ReadByte(&return_value, i);
				smb_return = return_value;
				if(smb_return!=0xff){
		  			c = (int)(1.35E6/(smb_return*fandiv[i-0x28]));
		  			if (c > 0) {
		    			digit = c % 10;
		    			copyXPMArea((digit<<2)+(digit<<1),64,6,9,51,((i-0x28)*8)+33);
		    			if (c > 9) {
		      			digit = ((c % 100) - digit) / 10;
		      			copyXPMArea((digit<<2)+(digit<<1),64,6,9,45,((i-0x28)*8)+33);
		      			if (c > 99) {
								digit = ((c % 1000) - digit) / 100;
								copyXPMArea((digit<<2)+(digit<<1),64,6,9,39,((i-0x28)*8)+33);
								if (c > 999) {
			  						digit = (c - (c % 1000)) / 1000;
			  						copyXPMArea((digit<<2)+(digit<<1),64,6,9,33,((i-0x28)*8)+33);
								}
		      			}
		    			}
		  			}
				}
				copyXPMArea(35,74,1,1,45,40); // Erase Decimal point from previous voltage display (if any)
	      }
	   } else if (data_display == D_VOLTAGES) {
	     	switch (voltage_type) {
	      	   case D_CORE:
	      	   case D_IO:
	      	   case D_P5:
	      	   case D_P12:
			DisplayVoltages(o_volts);
			break;	
	      	   case D_N5:
	      	   case D_N12:
			DisplayVoltages(o_volts);
			copyXPMArea(35,73,3,1,27,37); // Negative Beside Reading
			break;
	        }
	   } else if (data_display == D_TEMPS) {

	      // Display panel for single or dual cpus.

	      DisplayCPUTemps(1, cpu0_temp);
	      DisplayCPUTemps(2, cpu1_temp);	      
	      switch (scale) {
                case CELCIUS:
                copyXPMArea(61,74,5,7,52,34);
		copyXPMArea(61,74,5,7,52,42);
                break;
                case FAHRENHEIT:
                copyXPMArea(61,65,5,7,52,34);
		copyXPMArea(61,65,5,7,52,42);
                break;
                case KELVIN:
                copyXPMArea(71,65,5,7,52,34);
		copyXPMArea(71,65,5,7,52,42);
              }
              copyXPMArea(35,74,1,1,45,40); // Erase Decimal point from previous Voltage Display.
	   }

	if (sound_alarm && !ignore)
		Alarm(odd_test, mb_return, tbar_colormap);
			
   	if (odd_test) {
	   if (ignore)
	      copyXPMArea(52,73,2,2,24,6);
	   else
      	      copyXPMArea(50,73,2,2,24,6);
      	   odd_test = FALSE;
   	} else if (!odd_test) {
	   if (ignore)
	      copyXPMArea(54,73,2,2,24,6);
	   else
      	      copyXPMArea(48,73,2,2,24,6);
           odd_test = TRUE;
   	}
   	RedrawWindow();
        usleep(delay);
	} 	
}
